package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class ContainerStateWaitingBuilder extends ContainerStateWaitingFluentImpl<ContainerStateWaitingBuilder> implements VisitableBuilder<ContainerStateWaiting,ContainerStateWaitingBuilder>{

    ContainerStateWaitingFluent<?> fluent;
    Boolean validationEnabled;

    public ContainerStateWaitingBuilder(){
            this(true);
    }
    public ContainerStateWaitingBuilder(Boolean validationEnabled){
            this(new ContainerStateWaiting(), validationEnabled);
    }
    public ContainerStateWaitingBuilder(ContainerStateWaitingFluent<?> fluent){
            this(fluent, true);
    }
    public ContainerStateWaitingBuilder(ContainerStateWaitingFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new ContainerStateWaiting(), validationEnabled);
    }
    public ContainerStateWaitingBuilder(ContainerStateWaitingFluent<?> fluent,ContainerStateWaiting instance){
            this(fluent, instance, true);
    }
    public ContainerStateWaitingBuilder(ContainerStateWaitingFluent<?> fluent,ContainerStateWaiting instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withMessage(instance.getMessage()); 
            fluent.withReason(instance.getReason()); 
            this.validationEnabled = validationEnabled; 
    }
    public ContainerStateWaitingBuilder(ContainerStateWaiting instance){
            this(instance,true);
    }
    public ContainerStateWaitingBuilder(ContainerStateWaiting instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withMessage(instance.getMessage()); 
            this.withReason(instance.getReason()); 
            this.validationEnabled = validationEnabled; 
    }

    public ContainerStateWaiting build(){
            ContainerStateWaiting buildable = new ContainerStateWaiting(fluent.getMessage(),fluent.getReason());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            ContainerStateWaitingBuilder that = (ContainerStateWaitingBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
