package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class DeleteOptionsBuilder extends DeleteOptionsFluentImpl<DeleteOptionsBuilder> implements VisitableBuilder<DeleteOptions,DeleteOptionsBuilder>{

    DeleteOptionsFluent<?> fluent;
    Boolean validationEnabled;

    public DeleteOptionsBuilder(){
            this(true);
    }
    public DeleteOptionsBuilder(Boolean validationEnabled){
            this(new DeleteOptions(), validationEnabled);
    }
    public DeleteOptionsBuilder(DeleteOptionsFluent<?> fluent){
            this(fluent, true);
    }
    public DeleteOptionsBuilder(DeleteOptionsFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new DeleteOptions(), validationEnabled);
    }
    public DeleteOptionsBuilder(DeleteOptionsFluent<?> fluent,DeleteOptions instance){
            this(fluent, instance, true);
    }
    public DeleteOptionsBuilder(DeleteOptionsFluent<?> fluent,DeleteOptions instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withApiVersion(instance.getApiVersion()); 
            fluent.withGracePeriodSeconds(instance.getGracePeriodSeconds()); 
            fluent.withKind(instance.getKind()); 
            fluent.withOrphanDependents(instance.getOrphanDependents()); 
            fluent.withPreconditions(instance.getPreconditions()); 
            this.validationEnabled = validationEnabled; 
    }
    public DeleteOptionsBuilder(DeleteOptions instance){
            this(instance,true);
    }
    public DeleteOptionsBuilder(DeleteOptions instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withApiVersion(instance.getApiVersion()); 
            this.withGracePeriodSeconds(instance.getGracePeriodSeconds()); 
            this.withKind(instance.getKind()); 
            this.withOrphanDependents(instance.getOrphanDependents()); 
            this.withPreconditions(instance.getPreconditions()); 
            this.validationEnabled = validationEnabled; 
    }

    public DeleteOptions build(){
            DeleteOptions buildable = new DeleteOptions(fluent.getApiVersion(),fluent.getGracePeriodSeconds(),fluent.getKind(),fluent.isOrphanDependents(),fluent.getPreconditions());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            DeleteOptionsBuilder that = (DeleteOptionsBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
