package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class SecretKeySelectorBuilder extends SecretKeySelectorFluentImpl<SecretKeySelectorBuilder> implements VisitableBuilder<SecretKeySelector,SecretKeySelectorBuilder>{

    SecretKeySelectorFluent<?> fluent;
    Boolean validationEnabled;

    public SecretKeySelectorBuilder(){
            this(true);
    }
    public SecretKeySelectorBuilder(Boolean validationEnabled){
            this(new SecretKeySelector(), validationEnabled);
    }
    public SecretKeySelectorBuilder(SecretKeySelectorFluent<?> fluent){
            this(fluent, true);
    }
    public SecretKeySelectorBuilder(SecretKeySelectorFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new SecretKeySelector(), validationEnabled);
    }
    public SecretKeySelectorBuilder(SecretKeySelectorFluent<?> fluent,SecretKeySelector instance){
            this(fluent, instance, true);
    }
    public SecretKeySelectorBuilder(SecretKeySelectorFluent<?> fluent,SecretKeySelector instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withKey(instance.getKey()); 
            fluent.withName(instance.getName()); 
            this.validationEnabled = validationEnabled; 
    }
    public SecretKeySelectorBuilder(SecretKeySelector instance){
            this(instance,true);
    }
    public SecretKeySelectorBuilder(SecretKeySelector instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withKey(instance.getKey()); 
            this.withName(instance.getName()); 
            this.validationEnabled = validationEnabled; 
    }

    public SecretKeySelector build(){
            SecretKeySelector buildable = new SecretKeySelector(fluent.getKey(),fluent.getName());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            SecretKeySelectorBuilder that = (SecretKeySelectorBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
