package io.fabric8.kubernetes.api.model.extensions;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class JobStatusBuilder extends JobStatusFluentImpl<JobStatusBuilder> implements VisitableBuilder<JobStatus,JobStatusBuilder>{

    JobStatusFluent<?> fluent;
    Boolean validationEnabled;

    public JobStatusBuilder(){
            this(true);
    }
    public JobStatusBuilder(Boolean validationEnabled){
            this(new JobStatus(), validationEnabled);
    }
    public JobStatusBuilder(JobStatusFluent<?> fluent){
            this(fluent, true);
    }
    public JobStatusBuilder(JobStatusFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new JobStatus(), validationEnabled);
    }
    public JobStatusBuilder(JobStatusFluent<?> fluent,JobStatus instance){
            this(fluent, instance, true);
    }
    public JobStatusBuilder(JobStatusFluent<?> fluent,JobStatus instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withActive(instance.getActive()); 
            fluent.withCompletionTime(instance.getCompletionTime()); 
            fluent.withConditions(instance.getConditions()); 
            fluent.withFailed(instance.getFailed()); 
            fluent.withStartTime(instance.getStartTime()); 
            fluent.withSucceeded(instance.getSucceeded()); 
            this.validationEnabled = validationEnabled; 
    }
    public JobStatusBuilder(JobStatus instance){
            this(instance,true);
    }
    public JobStatusBuilder(JobStatus instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withActive(instance.getActive()); 
            this.withCompletionTime(instance.getCompletionTime()); 
            this.withConditions(instance.getConditions()); 
            this.withFailed(instance.getFailed()); 
            this.withStartTime(instance.getStartTime()); 
            this.withSucceeded(instance.getSucceeded()); 
            this.validationEnabled = validationEnabled; 
    }

    public JobStatus build(){
            JobStatus buildable = new JobStatus(fluent.getActive(),fluent.getCompletionTime(),fluent.getConditions(),fluent.getFailed(),fluent.getStartTime(),fluent.getSucceeded());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            JobStatusBuilder that = (JobStatusBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
