package io.fabric8.openshift.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class ImageStreamBuilder extends ImageStreamFluentImpl<ImageStreamBuilder> implements VisitableBuilder<ImageStream,ImageStreamBuilder>{

    ImageStreamFluent<?> fluent;
    Boolean validationEnabled;

    public ImageStreamBuilder(){
            this(true);
    }
    public ImageStreamBuilder(Boolean validationEnabled){
            this(new ImageStream(), validationEnabled);
    }
    public ImageStreamBuilder(ImageStreamFluent<?> fluent){
            this(fluent, true);
    }
    public ImageStreamBuilder(ImageStreamFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new ImageStream(), validationEnabled);
    }
    public ImageStreamBuilder(ImageStreamFluent<?> fluent,ImageStream instance){
            this(fluent, instance, true);
    }
    public ImageStreamBuilder(ImageStreamFluent<?> fluent,ImageStream instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withApiVersion(instance.getApiVersion()); 
            fluent.withKind(instance.getKind()); 
            fluent.withMetadata(instance.getMetadata()); 
            fluent.withSpec(instance.getSpec()); 
            fluent.withStatus(instance.getStatus()); 
            this.validationEnabled = validationEnabled; 
    }
    public ImageStreamBuilder(ImageStream instance){
            this(instance,true);
    }
    public ImageStreamBuilder(ImageStream instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withApiVersion(instance.getApiVersion()); 
            this.withKind(instance.getKind()); 
            this.withMetadata(instance.getMetadata()); 
            this.withSpec(instance.getSpec()); 
            this.withStatus(instance.getStatus()); 
            this.validationEnabled = validationEnabled; 
    }

    public ImageStream build(){
            ImageStream buildable = new ImageStream(fluent.getApiVersion(),fluent.getKind(),fluent.getMetadata(),fluent.getSpec(),fluent.getStatus());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            ImageStreamBuilder that = (ImageStreamBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
