package io.fabric8.openshift.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class RouteTargetReferenceBuilder extends RouteTargetReferenceFluentImpl<RouteTargetReferenceBuilder> implements VisitableBuilder<RouteTargetReference,RouteTargetReferenceBuilder>{

    RouteTargetReferenceFluent<?> fluent;
    Boolean validationEnabled;

    public RouteTargetReferenceBuilder(){
            this(true);
    }
    public RouteTargetReferenceBuilder(Boolean validationEnabled){
            this(new RouteTargetReference(), validationEnabled);
    }
    public RouteTargetReferenceBuilder(RouteTargetReferenceFluent<?> fluent){
            this(fluent, true);
    }
    public RouteTargetReferenceBuilder(RouteTargetReferenceFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new RouteTargetReference(), validationEnabled);
    }
    public RouteTargetReferenceBuilder(RouteTargetReferenceFluent<?> fluent,RouteTargetReference instance){
            this(fluent, instance, true);
    }
    public RouteTargetReferenceBuilder(RouteTargetReferenceFluent<?> fluent,RouteTargetReference instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withKind(instance.getKind()); 
            fluent.withName(instance.getName()); 
            fluent.withWeight(instance.getWeight()); 
            this.validationEnabled = validationEnabled; 
    }
    public RouteTargetReferenceBuilder(RouteTargetReference instance){
            this(instance,true);
    }
    public RouteTargetReferenceBuilder(RouteTargetReference instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withKind(instance.getKind()); 
            this.withName(instance.getName()); 
            this.withWeight(instance.getWeight()); 
            this.validationEnabled = validationEnabled; 
    }

    public RouteTargetReference build(){
            RouteTargetReference buildable = new RouteTargetReference(fluent.getKind(),fluent.getName(),fluent.getWeight());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            RouteTargetReferenceBuilder that = (RouteTargetReferenceBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
