package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class NamedExtensionBuilder extends NamedExtensionFluentImpl<NamedExtensionBuilder> implements VisitableBuilder<NamedExtension,NamedExtensionBuilder>{

    NamedExtensionFluent<?> fluent;
    Boolean validationEnabled;

    public NamedExtensionBuilder(){
            this(true);
    }
    public NamedExtensionBuilder(Boolean validationEnabled){
            this(new NamedExtension(), validationEnabled);
    }
    public NamedExtensionBuilder(NamedExtensionFluent<?> fluent){
            this(fluent, true);
    }
    public NamedExtensionBuilder(NamedExtensionFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new NamedExtension(), validationEnabled);
    }
    public NamedExtensionBuilder(NamedExtensionFluent<?> fluent,NamedExtension instance){
            this(fluent, instance, true);
    }
    public NamedExtensionBuilder(NamedExtensionFluent<?> fluent,NamedExtension instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withExtension(instance.getExtension()); 
            fluent.withName(instance.getName()); 
            this.validationEnabled = validationEnabled; 
    }
    public NamedExtensionBuilder(NamedExtension instance){
            this(instance,true);
    }
    public NamedExtensionBuilder(NamedExtension instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withExtension(instance.getExtension()); 
            this.withName(instance.getName()); 
            this.validationEnabled = validationEnabled; 
    }

    public NamedExtension build(){
            NamedExtension buildable = new NamedExtension(fluent.getExtension(),fluent.getName());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            NamedExtensionBuilder that = (NamedExtensionBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
